@@ -20,8 +20,11 @@
 
 """Quadratic programming solvers in Python with a unified API"""
 
+from .problem import Problem
+from .solution import Solution
 from .solve_ls import solve_ls
-from .solve_qp import solve_qp, solve_safer_qp
+from .solve_qp import solve_problem, solve_qp
+from .solve_safer_qp import solve_safer_qp
 from .solvers import (
     available_solvers,
     cvxopt_solve_qp,
@@ -40,9 +43,11 @@ from .solvers import (
 )
 from .utils import print_matrix_vector
 
-__version__ = "2.6.0"
+__version__ = "2.7.1"
 
 __all__ = [
+    "Problem",
+    "Solution",
     "__version__",
     "available_solvers",
     "cvxopt_solve_qp",
@@ -59,6 +64,7 @@ __all__ = [
     "quadprog_solve_qp",
     "scs_solve_qp",
     "solve_ls",
+    "solve_problem",
     "solve_qp",
     "solve_safer_qp",
     "sparse_solvers",
