@@ -25,15 +25,29 @@ from typing import Any, Callable, Dict, Optional, Union
 from numpy import ndarray
 from scipy.sparse import csc_matrix
 
+from ..problem import Problem
+from ..solution import Solution
+
 available_solvers = []
 dense_solvers = []
 solve_function: Dict[str, Any] = {}
 sparse_solvers = []
 
-
 # CVXOPT
 # ======
 
+cvxopt_solve_problem: Optional[
+    Callable[
+        [
+            Problem,
+            Optional[str],
+            Optional[ndarray],
+            bool,
+        ],
+        Solution,
+    ]
+] = None
+
 cvxopt_solve_qp: Optional[
     Callable[
         [
@@ -54,9 +68,9 @@ cvxopt_solve_qp: Optional[
 ] = None
 
 try:
-    from .cvxopt_ import cvxopt_solve_qp
+    from .cvxopt_ import cvxopt_solve_problem, cvxopt_solve_qp
 
-    solve_function["cvxopt"] = cvxopt_solve_qp
+    solve_function["cvxopt"] = cvxopt_solve_problem
     available_solvers.append("cvxopt")
     dense_solvers.append("cvxopt")
     sparse_solvers.append("cvxopt")
@@ -67,6 +81,17 @@ except ImportError:
 # ECOS
 # ====
 
+ecos_solve_problem: Optional[
+    Callable[
+        [
+            Problem,
+            Optional[ndarray],
+            bool,
+        ],
+        Solution,
+    ]
+] = None
+
 ecos_solve_qp: Optional[
     Callable[
         [
@@ -86,9 +111,9 @@ ecos_solve_qp: Optional[
 ] = None
 
 try:
-    from .ecos_ import ecos_solve_qp
+    from .ecos_ import ecos_solve_problem, ecos_solve_qp
 
-    solve_function["ecos"] = ecos_solve_qp
+    solve_function["ecos"] = ecos_solve_problem
     available_solvers.append("ecos")
     dense_solvers.append("ecos")  # considered dense as it calls cholesky(P)
 except ImportError:
@@ -98,6 +123,17 @@ except ImportError:
 # Gurobi
 # ======
 
+gurobi_solve_problem: Optional[
+    Callable[
+        [
+            Problem,
+            Optional[ndarray],
+            bool,
+        ],
+        Solution,
+    ]
+] = None
+
 gurobi_solve_qp: Optional[
     Callable[
         [
@@ -117,9 +153,9 @@ gurobi_solve_qp: Optional[
 ] = None
 
 try:
-    from .gurobi_ import gurobi_solve_qp
+    from .gurobi_ import gurobi_solve_problem, gurobi_solve_qp
 
-    solve_function["gurobi"] = gurobi_solve_qp
+    solve_function["gurobi"] = gurobi_solve_problem
     available_solvers.append("gurobi")
     sparse_solvers.append("gurobi")
 except ImportError:
@@ -129,6 +165,17 @@ except ImportError:
 # HiGHS
 # =====
 
+highs_solve_problem: Optional[
+    Callable[
+        [
+            Problem,
+            Optional[ndarray],
+            bool,
+        ],
+        Solution,
+    ]
+] = None
+
 highs_solve_qp: Optional[
     Callable[
         [
@@ -148,9 +195,9 @@ highs_solve_qp: Optional[
 ] = None
 
 try:
-    from .highs_ import highs_solve_qp
+    from .highs_ import highs_solve_problem, highs_solve_qp
 
-    solve_function["highs"] = highs_solve_qp
+    solve_function["highs"] = highs_solve_problem
     available_solvers.append("highs")
     sparse_solvers.append("highs")
 except ImportError:
@@ -160,6 +207,17 @@ except ImportError:
 # MOSEK
 # =====
 
+mosek_solve_problem: Optional[
+    Callable[
+        [
+            Problem,
+            Optional[ndarray],
+            bool,
+        ],
+        Solution,
+    ]
+] = None
+
 mosek_solve_qp: Optional[
     Callable[
         [
@@ -179,9 +237,9 @@ mosek_solve_qp: Optional[
 ] = None
 
 try:
-    from .mosek_ import mosek_solve_qp
+    from .mosek_ import mosek_solve_problem, mosek_solve_qp
 
-    solve_function["mosek"] = mosek_solve_qp
+    solve_function["mosek"] = mosek_solve_problem
     available_solvers.append("mosek")
     sparse_solvers.append("mosek")
 except ImportError:
@@ -191,6 +249,17 @@ except ImportError:
 # OSQP
 # ====
 
+osqp_solve_problem: Optional[
+    Callable[
+        [
+            Problem,
+            Optional[ndarray],
+            bool,
+        ],
+        Solution,
+    ]
+] = None
+
 osqp_solve_qp: Optional[
     Callable[
         [
@@ -210,9 +279,9 @@ osqp_solve_qp: Optional[
 ] = None
 
 try:
-    from .osqp_ import osqp_solve_qp
+    from .osqp_ import osqp_solve_problem, osqp_solve_qp
 
-    solve_function["osqp"] = osqp_solve_qp
+    solve_function["osqp"] = osqp_solve_problem
     available_solvers.append("osqp")
     sparse_solvers.append("osqp")
 except ImportError:
@@ -241,10 +310,22 @@ proxqp_solve_qp: Optional[
     ]
 ] = None
 
-try:
-    from .proxqp_ import proxqp_solve_qp
+proxqp_solve_problem: Optional[
+    Callable[
+        [
+            Problem,
+            Optional[Union[ndarray, csc_matrix]],
+            bool,
+            Optional[str],
+        ],
+        Solution,
+    ]
+] = None
 
-    solve_function["proxqp"] = proxqp_solve_qp
+try:
+    from .proxqp_ import proxqp_solve_problem, proxqp_solve_qp
+
+    solve_function["proxqp"] = proxqp_solve_problem
     available_solvers.append("proxqp")
     dense_solvers.append("proxqp")
     sparse_solvers.append("proxqp")
@@ -255,6 +336,19 @@ except ImportError:
 # qpOASES
 # =======
 
+qpoases_solve_problem: Optional[
+    Callable[
+        [
+            Problem,
+            Optional[ndarray],
+            bool,
+            int,
+            Optional[float],
+        ],
+        Solution,
+    ]
+] = None
+
 qpoases_solve_qp: Optional[
     Callable[
         [
@@ -276,9 +370,9 @@ qpoases_solve_qp: Optional[
 ] = None
 
 try:
-    from .qpoases_ import qpoases_solve_qp
+    from .qpoases_ import qpoases_solve_problem, qpoases_solve_qp
 
-    solve_function["qpoases"] = qpoases_solve_qp
+    solve_function["qpoases"] = qpoases_solve_problem
     available_solvers.append("qpoases")
     dense_solvers.append("qpoases")
 except ImportError:
@@ -288,6 +382,17 @@ except ImportError:
 # qpSWIFT
 # =======
 
+qpswift_solve_problem: Optional[
+    Callable[
+        [
+            Problem,
+            Optional[ndarray],
+            bool,
+        ],
+        Solution,
+    ]
+] = None
+
 qpswift_solve_qp: Optional[
     Callable[
         [
@@ -307,9 +412,9 @@ qpswift_solve_qp: Optional[
 ] = None
 
 try:
-    from .qpswift_ import qpswift_solve_qp
+    from .qpswift_ import qpswift_solve_problem, qpswift_solve_qp
 
-    solve_function["qpswift"] = qpswift_solve_qp
+    solve_function["qpswift"] = qpswift_solve_problem
     available_solvers.append("qpswift")
     dense_solvers.append("qpswift")
 except ImportError:
@@ -337,10 +442,21 @@ quadprog_solve_qp: Optional[
     ]
 ] = None
 
-try:
-    from .quadprog_ import quadprog_solve_qp
+quadprog_solve_problem: Optional[
+    Callable[
+        [
+            Problem,
+            Optional[ndarray],
+            bool,
+        ],
+        Solution,
+    ]
+] = None
 
-    solve_function["quadprog"] = quadprog_solve_qp
+try:
+    from .quadprog_ import quadprog_solve_problem, quadprog_solve_qp
+
+    solve_function["quadprog"] = quadprog_solve_problem
     available_solvers.append("quadprog")
     dense_solvers.append("quadprog")
 except ImportError:
@@ -350,6 +466,17 @@ except ImportError:
 # SCS
 # ========
 
+scs_solve_problem: Optional[
+    Callable[
+        [
+            Problem,
+            Optional[ndarray],
+            bool,
+        ],
+        Solution,
+    ]
+] = None
+
 scs_solve_qp: Optional[
     Callable[
         [
@@ -369,9 +496,9 @@ scs_solve_qp: Optional[
 ] = None
 
 try:
-    from .scs_ import scs_solve_qp
+    from .scs_ import scs_solve_problem, scs_solve_qp
 
-    solve_function["scs"] = scs_solve_qp
+    solve_function["scs"] = scs_solve_problem
     available_solvers.append("scs")
     sparse_solvers.append("scs")
 except ImportError:
@@ -389,11 +516,16 @@ __all__ = [
     "available_solvers",
     "cvxopt_solve_qp",
     "dense_solvers",
+    "ecos_solve_qp",
     "gurobi_solve_qp",
+    "highs_solve_qp",
     "mosek_solve_qp",
     "osqp_solve_qp",
+    "proxqp_solve_qp",
     "qpoases_solve_qp",
+    "qpswift_solve_qp",
     "quadprog_solve_qp",
+    "scs_solve_qp",
     "solve_function",
     "sparse_solvers",
 ]
