@@ -29,6 +29,7 @@ using CVXOPT in some academic work, consider citing the corresponding report
 [Vandenberghe2010]_.
 """
 
+import warnings
 from typing import Dict, Optional, Union
 
 import cvxopt
@@ -36,12 +37,14 @@ import numpy as np
 import scipy.sparse as spa
 from cvxopt.solvers import qp
 
-from .conversions import linear_from_box_inequalities
+from ..conversions import linear_from_box_inequalities, split_dual_linear_box
+from ..problem import Problem
+from ..solution import Solution
 
 cvxopt.solvers.options["show_progress"] = False  # disable verbose output
 
 
-def to_cvxopt(
+def __to_cvxopt(
     M: Union[np.ndarray, spa.csc_matrix]
 ) -> Union[cvxopt.matrix, cvxopt.spmatrix]:
     """
@@ -65,61 +68,20 @@ def to_cvxopt(
     )
 
 
-def cvxopt_solve_qp(
-    P: Union[np.ndarray, spa.csc_matrix],
-    q: np.ndarray,
-    G: Optional[Union[np.ndarray, spa.csc_matrix]] = None,
-    h: Optional[np.ndarray] = None,
-    A: Optional[Union[np.ndarray, spa.csc_matrix]] = None,
-    b: Optional[np.ndarray] = None,
-    lb: Optional[np.ndarray] = None,
-    ub: Optional[np.ndarray] = None,
+def cvxopt_solve_problem(
+    problem: Problem,
     solver: Optional[str] = None,
     initvals: Optional[np.ndarray] = None,
     verbose: bool = False,
     **kwargs,
-) -> Optional[np.ndarray]:
+) -> Solution:
     """
-    Solve a Quadratic Program defined as:
-
-    .. math::
-
-        \\begin{split}\\begin{array}{ll}
-        \\mbox{minimize} &
-            \\frac{1}{2} x^T P x + q^T x \\\\
-        \\mbox{subject to}
-            & G x \\leq h                \\\\
-            & A x = b                    \\\\
-            & lb \\leq x \\leq ub
-        \\end{array}\\end{split}
-
-    using `CVXOPT <http://cvxopt.org/>`_.
+    Solve a quadratic program using `CVXOPT <http://cvxopt.org/>`_.
 
     Parameters
     ----------
-    P :
-        Symmetric quadratic-cost matrix. Together with :math:`A` and :math:`G`,
-        it should satisfy :math:`\\mathrm{rank}([P\\ A^T\\ G^T]) = n`, see the
-        rank assumptions below.
-    q :
-        Quadratic-cost vector.
-    G :
-        Linear inequality matrix. Together with :math:`P` and :math:`A`, it
-        should satisfy :math:`\\mathrm{rank}([P\\ A^T\\ G^T]) = n`, see the
-        rank assumptions below.
-    h :
-        Linear inequality vector.
-    A :
-        Linear equality constraint matrix. It needs to be full row rank, and
-        together with :math:`P` and :math:`G` satisfy
-        :math:`\\mathrm{rank}([P\\ A^T\\ G^T]) = n`. See the rank assumptions
-        below.
-    b :
-        Linear equality constraint vector.
-    lb :
-        Lower bound constraint vector.
-    ub :
-        Upper bound constraint vector.
+    problem :
+        Quadratic program to solve.
     solver :
         Set to 'mosek' to run MOSEK rather than CVXOPT.
     initvals :
@@ -182,27 +144,119 @@ def cvxopt_solve_qp(
     parameters. See also [tolerances]_ for a primer on the duality gap, primal
     and dual residuals.
     """
+    P, q, G, h, A, b, lb, ub = problem.unpack()
     if lb is not None or ub is not None:
         G, h = linear_from_box_inequalities(G, h, lb, ub)
 
-    args = [to_cvxopt(P), to_cvxopt(q)]
+    args = [__to_cvxopt(P), __to_cvxopt(q)]
     constraints = {"G": None, "h": None, "A": None, "b": None}
     if G is not None and h is not None:
-        constraints["G"] = to_cvxopt(G)
-        constraints["h"] = to_cvxopt(h)
+        constraints["G"] = __to_cvxopt(G)
+        constraints["h"] = __to_cvxopt(h)
     if A is not None and b is not None:
-        constraints["A"] = to_cvxopt(A)
-        constraints["b"] = to_cvxopt(b)
+        constraints["A"] = __to_cvxopt(A)
+        constraints["b"] = __to_cvxopt(b)
     initvals_dict: Optional[Dict[str, cvxopt.matrix]] = None
     if initvals is not None:
-        initvals_dict = {"x": to_cvxopt(initvals)}
+        initvals_dict = {"x": __to_cvxopt(initvals)}
 
     kwargs["show_progress"] = verbose
     original_options = cvxopt.solvers.options
     cvxopt.solvers.options = kwargs
-    solution = qp(*args, solver=solver, initvals=initvals_dict, **constraints)
+    res = qp(*args, solver=solver, initvals=initvals_dict, **constraints)
     cvxopt.solvers.options = original_options
 
-    if "optimal" not in solution["status"]:
-        return None
-    return np.array(solution["x"]).reshape((q.shape[0],))
+    solution = Solution(problem)
+    if "optimal" not in res["status"]:
+        return solution
+    solution.x = np.array(res["x"]).reshape((q.shape[0],))
+    if b is not None:
+        solution.y = np.array(res["y"]).reshape((b.shape[0],))
+    if h is not None:
+        z_cvx = np.array(res["z"]).reshape((h.shape[0],))
+        z, z_box = split_dual_linear_box(z_cvx, lb, ub)
+        solution.z = z
+        solution.z_box = z_box
+    solution.obj = res["primal objective"]
+    solution.extras = res
+    return solution
+
+
+def cvxopt_solve_qp(
+    P: Union[np.ndarray, spa.csc_matrix],
+    q: np.ndarray,
+    G: Optional[Union[np.ndarray, spa.csc_matrix]] = None,
+    h: Optional[np.ndarray] = None,
+    A: Optional[Union[np.ndarray, spa.csc_matrix]] = None,
+    b: Optional[np.ndarray] = None,
+    lb: Optional[np.ndarray] = None,
+    ub: Optional[np.ndarray] = None,
+    solver: Optional[str] = None,
+    initvals: Optional[np.ndarray] = None,
+    verbose: bool = False,
+    **kwargs,
+) -> Optional[np.ndarray]:
+    """
+    Solve a Quadratic Program defined as:
+
+    .. math::
+
+        \\begin{split}\\begin{array}{ll}
+            \\underset{x}{\\mbox{minimize}} &
+                \\frac{1}{2} x^T P x + q^T x \\\\
+            \\mbox{subject to}
+                & G x \\leq h                \\\\
+                & A x = b                    \\\\
+                & lb \\leq x \\leq ub
+        \\end{array}\\end{split}
+
+    using `CVXOPT <http://cvxopt.org/>`_.
+
+    Parameters
+    ----------
+    P :
+        Symmetric cost matrix. Together with :math:`A` and :math:`G`, it should
+        satisfy :math:`\\mathrm{rank}([P\\ A^T\\ G^T]) = n`, see the rank
+        assumptions below.
+    q :
+        Cost vector.
+    G :
+        Linear inequality matrix. Together with :math:`P` and :math:`A`, it
+        should satisfy :math:`\\mathrm{rank}([P\\ A^T\\ G^T]) = n`, see the
+        rank assumptions below.
+    h :
+        Linear inequality vector.
+    A :
+        Linear equality constraint matrix. It needs to be full row rank, and
+        together with :math:`P` and :math:`G` satisfy
+        :math:`\\mathrm{rank}([P\\ A^T\\ G^T]) = n`. See the rank assumptions
+        below.
+    b :
+        Linear equality constraint vector.
+    lb :
+        Lower bound constraint vector.
+    ub :
+        Upper bound constraint vector.
+    solver :
+        Set to 'mosek' to run MOSEK rather than CVXOPT.
+    initvals :
+        Warm-start guess vector.
+    verbose :
+        Set to `True` to print out extra information.
+
+    Returns
+    -------
+    :
+        Primal solution to the QP, if found, otherwise ``None``.
+    """
+    warnings.warn(
+        "The return type of this function will change "
+        "to qpsolvers.Solution in qpsolvers v3.0",
+        DeprecationWarning,
+        stacklevel=2,
+    )
+    problem = Problem(P, q, G, h, A, b, lb, ub)
+    solution = cvxopt_solve_problem(
+        problem, solver, initvals, verbose, **kwargs
+    )
+    return solution.x
