@@ -29,7 +29,7 @@ from numpy import array, ones
 from numpy.linalg import norm
 from scipy.sparse import csc_matrix
 
-from qpsolvers import solve_qp
+from qpsolvers.solvers import cvxopt_solve_qp
 
 from .problems import get_sd3310_problem
 
@@ -55,9 +55,9 @@ class TestCVXOPT(unittest.TestCase):
         Returns
         -------
         P :
-            Symmetric quadratic-cost matrix .
+            Symmetric cost matrix.
         q :
-            Quadratic-cost vector.
+            Cost vector.
         G :
             Linear inequality matrix.
         h :
@@ -79,7 +79,7 @@ class TestCVXOPT(unittest.TestCase):
         Test CVXOPT on a sparse problem.
         """
         P, q, G, h = self.get_sparse_problem()
-        x = solve_qp(P, q, G, h, solver="cvxopt")
+        x = cvxopt_solve_qp(P, q, G, h)
         self.assertIsNotNone(x)
         known_solution = array([2.0] * 149 + [3.0])
         sol_tolerance = 1e-2  # aouch, not great!
@@ -90,15 +90,14 @@ class TestCVXOPT(unittest.TestCase):
         """
         Call CVXOPT with various solver-specific settings.
         """
-        P, q, G, h, A, b = get_sd3310_problem()
-        x = solve_qp(
-            P,
-            q,
-            G,
-            h,
-            A,
-            b,
-            solver="cvxopt",
+        problem = get_sd3310_problem()
+        x = cvxopt_solve_qp(
+            problem.P,
+            problem.q,
+            problem.G,
+            problem.h,
+            problem.A,
+            problem.b,
             maxiters=10,
             abstol=1e-1,
             reltol=1e-1,
