@@ -20,8 +20,6 @@
 
 import unittest
 
-from qpsolvers import solve_qp
-
 from .problems import get_sd3310_problem
 
 try:
@@ -37,15 +35,14 @@ try:
             """
             Try the dense backend.
             """
-            P, q, G, h, A, b = get_sd3310_problem()
-            solve_qp(
-                P,
-                q,
-                G,
-                h,
-                A,
-                b,
-                solver="proxqp",
+            problem = get_sd3310_problem()
+            proxqp_solve_qp(
+                problem.P,
+                problem.q,
+                problem.G,
+                problem.h,
+                problem.A,
+                problem.b,
                 backend="dense",
             )
 
@@ -53,15 +50,14 @@ try:
             """
             Try the sparse backend.
             """
-            P, q, G, h, A, b = get_sd3310_problem()
-            solve_qp(
-                P,
-                q,
-                G,
-                h,
-                A,
-                b,
-                solver="proxqp",
+            problem = get_sd3310_problem()
+            proxqp_solve_qp(
+                problem.P,
+                problem.q,
+                problem.G,
+                problem.h,
+                problem.A,
+                problem.b,
                 backend="sparse",
             )
 
@@ -69,16 +65,15 @@ try:
             """
             Exception raised when asking for an invalid backend.
             """
-            P, q, G, h, A, b = get_sd3310_problem()
+            problem = get_sd3310_problem()
             with self.assertRaises(ValueError):
-                solve_qp(
-                    P,
-                    q,
-                    G,
-                    h,
-                    A,
-                    b,
-                    solver="proxqp",
+                proxqp_solve_qp(
+                    problem.P,
+                    problem.q,
+                    problem.G,
+                    problem.h,
+                    problem.A,
+                    problem.b,
                     backend="invalid",
                 )
 
@@ -87,29 +82,34 @@ try:
             Raise an exception when two warm-start values are provided at the
             same time.
             """
-            P, q, G, h, A, b = get_sd3310_problem()
+            problem = get_sd3310_problem()
             with self.assertRaises(ValueError):
-                solve_qp(
-                    P,
-                    q,
-                    G,
-                    h,
-                    A,
-                    b,
-                    solver="proxqp",
-                    initvals=q,
-                    x=q,
+                proxqp_solve_qp(
+                    problem.P,
+                    problem.q,
+                    problem.G,
+                    problem.h,
+                    problem.A,
+                    problem.b,
+                    initvals=problem.q,
+                    x=problem.q,
                 )
 
         def test_invalid_inequalities(self):
             """
             Raise an exception in an implementation-dependent inconsistent set
-            of parameters. This won't happen when the function is called by
-            `solve_qp`, but it may happen when it is called directly.
+            of parameters. This may happen when :func:`proxqp_solve_qp` it is
+            called directly.
             """
-            P, q, G, _, _, _ = get_sd3310_problem()
+            problem = get_sd3310_problem()
             with self.assertRaises(ValueError):
-                proxqp_solve_qp(P, q, G=G, h=None, lb=q)
+                proxqp_solve_qp(
+                    problem.P,
+                    problem.q,
+                    G=problem.G,
+                    h=None,
+                    lb=problem.q,
+                )
 
 
 except ImportError:  # ProxSuite not installed
